@@ -5,6 +5,8 @@
 #' @param object Model object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param ... not used
 #' @section n-step ahead forecasting VAR(p):
 #' See pp35 of Lütkepohl (2007).
@@ -50,10 +52,25 @@
 #' @references Lütkepohl, H. (2007). *New Introduction to Multiple Time Series Analysis*. Springer Publishing.
 #' @name predict
 #' @importFrom stats qnorm
+#' @importFrom utils tail
 #' @order 1
 #' @export
-predict.varlse <- function(object, n_ahead, level = .05, ...) {
-  pred_res <- forecast_var(object, n_ahead)
+predict.varlse <- function(object, n_ahead, level = .05, newxreg, ...) {
+  if (!is.null(eval.parent(object$call$exogen))) {
+    newxreg <- validate_newxreg(newxreg = newxreg, n_ahead = n_ahead)
+    pred_res <- forecast_varx(
+      response = object$y0,
+      coef_mat = object$coefficients[-object$exogen_id, ],
+      lag = object$p,
+      step = n_ahead,
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_coef = object$coefficients[object$exogen_id, ],
+      exogen_lag = object$s
+    )
+  } else {
+    pred_res <- forecast_var(object, n_ahead)
+  }
   colnames(pred_res) <- colnames(object$y0)
   SE <- 
     compute_covmse(object, n_ahead) |> # concatenated matrix
@@ -82,6 +99,8 @@ predict.varlse <- function(object, n_ahead, level = .05, ...) {
 #' @param object A `vharlse` object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param ... not used
 #' @section n-step ahead forecasting VHAR:
 #' Let \eqn{T_{HAR}} is VHAR linear transformation matrix.
@@ -105,13 +124,36 @@ predict.varlse <- function(object, n_ahead, level = .05, ...) {
 #' @importFrom stats qnorm
 #' @order 1
 #' @export
-predict.vharlse <- function(object, n_ahead, level = .05, ...) {
-  pred_res <- forecast_vhar(object, n_ahead)
+predict.vharlse <- function(object, n_ahead, level = .05, newxreg, ...) {
+  if (!is.null(eval.parent(object$call$exogen))) {
+    if (missing(newxreg) || is.null(newxreg)) {
+      stop("'newxreg' should be supplied when using VHARX model.")
+    }
+    if (!is.matrix(newxreg)) {
+      newxreg <- as.matrix(newxreg)
+    }
+    if (nrow(newxreg) != n_ahead) {
+      stop("Wrong row number of 'newxreg'")
+    }
+    pred_res <- forecast_harx(
+      response = object$y0,
+      coef_mat = object$coefficients[-object$exogen_id, ],
+      week = object$week,
+      month = object$month,
+      step = n_ahead,
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_coef = object$coefficients[object$exogen_id, ],
+      exogen_lag = object$s
+    )
+  } else {
+    pred_res <- forecast_vhar(object, n_ahead)
+  }
   colnames(pred_res) <- colnames(object$y0)
-  SE <- 
+  SE <-
     compute_covmse_har(object, n_ahead) |> # concatenated matrix
     split.data.frame(gl(n_ahead, object$m)) |> # list of forecast MSE covariance matrix
-    sapply(diag) |> 
+    sapply(diag) |>
     t() # extract only diagonal element to compute CIs
   SE <- sqrt(SE)
   colnames(SE) <- colnames(object$y0)
@@ -378,6 +420,8 @@ predict.bvarflat <- function(object, n_ahead, n_iter = 100L, level = .05, num_th
 #' @param object Model object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param stable `r lifecycle::badge("experimental")` Filter only stable coefficient draws in MCMC records.
 #' @param num_thread Number of threads
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
@@ -390,7 +434,7 @@ predict.bvarflat <- function(object, n_ahead, n_iter = 100L, level = .05, num_th
 #' @importFrom stats median
 #' @order 1
 #' @export
-predict.bvarldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_thread = 1, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
+predict.bvarldlt <- function(object, n_ahead, level = .05, newxreg, stable = FALSE, num_thread = 1, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
   dim_data <- object$m
   num_chains <- object$chain
   alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha"))
@@ -419,39 +463,57 @@ predict.bvarldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_t
   if (num_thread > num_chains && num_chains != 1) {
     warning("'num_thread' > 'num_chains' will not use every thread. Specify as 'num_thread' <= 'num_chains'.")
   }
-  prior_nm <- object$spec$prior
+  # prior_nm <- object$spec$prior
   # ci_lev <- NULL
   ci_lev <- 0
   if (is.numeric(sparse)) {
     ci_lev <- sparse
     sparse <- FALSE
-    prior_nm <- "ci"
+    # prior_nm <- "ci"
   }
   fit_ls <- get_records(object, TRUE)
-  prior_type <- switch(prior_nm,
-    "ci" = 0,
-    "Minnesota" = 1,
-    "SSVS" = 2,
-    "Horseshoe" = 3,
-    "MN_Hierarchical" = 4,
-    "NG" = 5,
-    "DL" = 6,
-    "GDP" = 7
-  )
-  pred_res <- forecast_bvarldlt(
-    num_chains,
-    object$p,
-    n_ahead,
-    object$y,
-    sparse,
-    ci_lev,
-    fit_ls,
-    prior_type,
-    sample.int(.Machine$integer.max, size = num_chains),
-    object$type == "const",
-    stable,
-    num_thread
-  )
+  # prior_type <- switch(prior_nm,
+  #   "ci" = 0,
+  #   "Minnesota" = 1,
+  #   "SSVS" = 2,
+  #   "Horseshoe" = 3,
+  #   "MN_Hierarchical" = 4,
+  #   "NG" = 5,
+  #   "DL" = 6,
+  #   "GDP" = 7
+  # )
+  if (!is.null(eval.parent(object$call$exogen))) {
+    newxreg <- validate_newxreg(newxreg = newxreg, n_ahead = n_ahead)
+    pred_res <- forecast_bvarxldlt(
+      num_chains = num_chains,
+      var_lag = object$p,
+      step = n_ahead,
+      response_mat = object$y,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_lag = object$s,
+      stable = stable,
+      nthreads = num_thread
+    )
+  } else {
+    pred_res <- forecast_bvarldlt(
+      num_chains = num_chains,
+      var_lag = object$p,
+      step = n_ahead,
+      response_mat = object$y,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      stable = stable,
+      nthreads = num_thread
+    )
+  }
   var_names <- colnames(object$y0)
   # Predictive distribution------------------------------------
   num_draw <- nrow(alpha_record) # concatenate multiple chains
@@ -490,6 +552,8 @@ predict.bvarldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_t
 #' @param object Model object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param stable `r lifecycle::badge("experimental")` Filter only stable coefficient draws in MCMC records.
 #' @param num_thread Number of threads
 #' @param sparse `r lifecycle::badge("experimental")` Apply restriction. By default, `FALSE`.
@@ -500,7 +564,7 @@ predict.bvarldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_t
 #' @importFrom posterior subset_draws as_draws_matrix
 #' @order 1
 #' @export
-predict.bvharldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_thread = 1, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
+predict.bvharldlt <- function(object, n_ahead, level = .05, newxreg, stable = FALSE, num_thread = 1, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
   dim_data <- object$m
   num_chains <- object$chain
   phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi"))
@@ -530,39 +594,58 @@ predict.bvharldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_
   if (num_thread > num_chains && num_chains != 1) {
     warning("'num_thread' > 'num_chains' will not use every thread. Specify as 'num_thread' <= 'num_chains'.")
   }
-  prior_nm <- object$spec$prior
+  # prior_nm <- object$spec$prior
   ci_lev <- 0
   if (is.numeric(sparse)) {
     ci_lev <- sparse
     sparse <- FALSE
-    prior_nm <- "ci"
+    # prior_nm <- "ci"
   }
   fit_ls <- get_records(object, TRUE)
-  prior_type <- switch(prior_nm,
-    "ci" = 0,
-    "Minnesota" = 1,
-    "SSVS" = 2,
-    "Horseshoe" = 3,
-    "MN_Hierarchical" = 4,
-    "NG" = 5,
-    "DL" = 6,
-    "GDP" = 7
-  )
-  pred_res <- forecast_bvharldlt(
-    num_chains,
-    object$month,
-    n_ahead,
-    object$y,
-    object$HARtrans,
-    sparse,
-    ci_lev,
-    fit_ls,
-    prior_type,
-    sample.int(.Machine$integer.max, size = num_chains),
-    object$type == "const",
-    stable,
-    num_thread
-  )
+  # prior_type <- switch(prior_nm,
+  #   "ci" = 0,
+  #   "Minnesota" = 1,
+  #   "SSVS" = 2,
+  #   "Horseshoe" = 3,
+  #   "MN_Hierarchical" = 4,
+  #   "NG" = 5,
+  #   "DL" = 6,
+  #   "GDP" = 7
+  # )
+  if (!is.null(eval.parent(object$call$exogen))) {
+    newxreg <- validate_newxreg(newxreg = newxreg, n_ahead = n_ahead)
+    pred_res <- forecast_bvharxldlt(
+      num_chains = num_chains,
+      month = object$month,
+      step = n_ahead,
+      response_mat = object$y,
+      HARtrans = object$HARtrans,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_lag = object$s,
+      stable = stable,
+      nthreads = num_thread
+    )
+  } else {
+    pred_res <- forecast_bvharldlt(
+      num_chains = num_chains,
+      month = object$month,
+      step = n_ahead,
+      response_mat = object$y,
+      HARtrans = object$HARtrans,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      stable = stable,
+      nthreads = num_thread
+    )
+  }
   var_names <- colnames(object$y0)
   # Predictive distribution------------------------------------
   num_draw <- nrow(phi_record) # concatenate multiple chains
@@ -601,6 +684,8 @@ predict.bvharldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_
 #' @param object Model object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param stable `r lifecycle::badge("experimental")` Filter only stable coefficient draws in MCMC records.
 #' @param num_thread Number of threads
 #' @param use_sv Use SV term
@@ -616,7 +701,7 @@ predict.bvharldlt <- function(object, n_ahead, level = .05, stable = FALSE, num_
 #' @importFrom posterior subset_draws as_draws_matrix
 #' @order 1
 #' @export
-predict.bvarsv <- function(object, n_ahead, level = .05, stable = FALSE, num_thread = 1, use_sv = TRUE, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
+predict.bvarsv <- function(object, n_ahead, level = .05, newxreg, stable = FALSE, num_thread = 1, use_sv = TRUE, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
   dim_data <- object$m
   num_chains <- object$chain
   alpha_record <- as_draws_matrix(subset_draws(object$param, variable = "alpha"))
@@ -645,40 +730,59 @@ predict.bvarsv <- function(object, n_ahead, level = .05, stable = FALSE, num_thr
   if (num_thread > num_chains && num_chains != 1) {
     warning("'num_thread' > 'num_chains' will not use every thread. Specify as 'num_thread' <= 'num_chains'.")
   }
-  prior_nm <- object$spec$prior
+  # prior_nm <- object$spec$prior
   # ci_lev <- NULL
   ci_lev <- 0
   if (is.numeric(sparse)) {
     ci_lev <- sparse
     sparse <- FALSE
-    prior_nm <- "ci"
+    # prior_nm <- "ci"
   }
   fit_ls <- get_records(object, TRUE)
-  prior_type <- switch(prior_nm,
-    "ci" = 0,
-    "Minnesota" = 1,
-    "SSVS" = 2,
-    "Horseshoe" = 3,
-    "MN_Hierarchical" = 4,
-    "NG" = 5,
-    "DL" = 6,
-    "GDP" = 7
-  )
-  pred_res <- forecast_bvarsv(
-    num_chains,
-    object$p,
-    n_ahead,
-    object$y,
-    use_sv,
-    sparse,
-    ci_lev,
-    fit_ls,
-    prior_type,
-    sample.int(.Machine$integer.max, size = num_chains),
-    object$type == "const",
-    stable,
-    num_thread
-  )
+  # prior_type <- switch(prior_nm,
+  #   "ci" = 0,
+  #   "Minnesota" = 1,
+  #   "SSVS" = 2,
+  #   "Horseshoe" = 3,
+  #   "MN_Hierarchical" = 4,
+  #   "NG" = 5,
+  #   "DL" = 6,
+  #   "GDP" = 7
+  # )
+  if (!is.null(eval.parent(object$call$exogen))) {
+    newxreg <- validate_newxreg(newxreg = newxreg, n_ahead = n_ahead)
+    pred_res <- forecast_bvarxsv(
+      num_chains = num_chains,
+      var_lag = object$p,
+      step = n_ahead,
+      response_mat = object$y,
+      sv = use_sv,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_lag = object$s,
+      stable = stable,
+      nthreads = num_thread
+    )
+  } else {
+    pred_res <- forecast_bvarsv(
+      num_chains = num_chains,
+      var_lag = object$p,
+      step = n_ahead,
+      response_mat = object$y,
+      sv = use_sv,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      stable = stable,
+      nthreads = num_thread
+    )
+  }
   var_names <- colnames(object$y0)
   # Predictive distribution------------------------------------
   num_draw <- nrow(alpha_record) # concatenate multiple chains
@@ -717,6 +821,8 @@ predict.bvarsv <- function(object, n_ahead, level = .05, stable = FALSE, num_thr
 #' @param object Model object
 #' @param n_ahead step to forecast
 #' @param level Specify alpha of confidence interval level 100(1 - alpha) percentage. By default, .05.
+#' @param newxreg New values for exogenous variables.
+#' Should have the same row numbers with `n_ahead`.
 #' @param stable `r lifecycle::badge("experimental")` Filter only stable coefficient draws in MCMC records.
 #' @param num_thread Number of threads
 #' @param use_sv Use SV term
@@ -728,7 +834,7 @@ predict.bvarsv <- function(object, n_ahead, level = .05, stable = FALSE, num_thr
 #' @importFrom posterior subset_draws as_draws_matrix
 #' @order 1
 #' @export
-predict.bvharsv <- function(object, n_ahead, level = .05, stable = FALSE, num_thread = 1, use_sv = TRUE, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
+predict.bvharsv <- function(object, n_ahead, level = .05, newxreg, stable = FALSE, num_thread = 1, use_sv = TRUE, sparse = FALSE, med = FALSE, warn = FALSE, ...) {
   dim_data <- object$m
   num_chains <- object$chain
   phi_record <- as_draws_matrix(subset_draws(object$param, variable = "phi"))
@@ -758,41 +864,60 @@ predict.bvharsv <- function(object, n_ahead, level = .05, stable = FALSE, num_th
   if (num_thread > num_chains && num_chains != 1) {
     warning("'num_thread' > 'num_chains' will not use every thread. Specify as 'num_thread' <= 'num_chains'.")
   }
-  prior_nm <- object$spec$prior
+  # prior_nm <- object$spec$prior
   # ci_lev <- NULL
   ci_lev <- 0
   if (is.numeric(sparse)) {
     ci_lev <- sparse
     sparse <- FALSE
-    prior_nm <- "ci"
+    # prior_nm <- "ci"
   }
   fit_ls <- get_records(object, TRUE)
-  prior_type <- switch(prior_nm,
-    "ci" = 0,
-    "Minnesota" = 1,
-    "SSVS" = 2,
-    "Horseshoe" = 3,
-    "MN_Hierarchical" = 4,
-    "NG" = 5,
-    "DL" = 6,
-    "GDP" = 7
-  )
-  pred_res <- forecast_bvharsv(
-    num_chains,
-    object$month,
-    n_ahead,
-    object$y,
-    object$HARtrans,
-    use_sv,
-    sparse,
-    ci_lev,
-    fit_ls,
-    prior_type,
-    sample.int(.Machine$integer.max, size = num_chains),
-    object$type == "const",
-    stable,
-    num_thread
-  )
+  # prior_type <- switch(prior_nm,
+  #   "ci" = 0,
+  #   "Minnesota" = 1,
+  #   "SSVS" = 2,
+  #   "Horseshoe" = 3,
+  #   "MN_Hierarchical" = 4,
+  #   "NG" = 5,
+  #   "DL" = 6,
+  #   "GDP" = 7
+  # )
+  if (!is.null(eval.parent(object$call$exogen))) {
+    newxreg <- validate_newxreg(newxreg = newxreg, n_ahead = n_ahead)
+    pred_res <- forecast_bvharxsv(
+      num_chains = num_chains,
+      month = object$month,
+      step = n_ahead,
+      response_mat = object$y,
+      HARtrans = object$HARtrans,
+      sv = use_sv,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      exogen = rbind(tail(object$exogen_data, object$s), newxreg),
+      exogen_lag = object$s,
+      stable = stable,
+      nthreads = num_thread
+    )
+  } else {
+    pred_res <- forecast_bvharsv(
+      num_chains = num_chains,
+      month = object$month,
+      step = n_ahead,
+      response_mat = object$y,
+      sv = use_sv,
+      sparse = sparse,
+      level = ci_lev,
+      fit_record = fit_ls,
+      seed_chain = sample.int(.Machine$integer.max, size = num_chains),
+      include_mean = object$type == "const",
+      stable = stable,
+      nthreads = num_thread
+    )
+  }
   var_names <- colnames(object$y0)
   # Predictive distribution------------------------------------
   num_draw <- nrow(phi_record) # concatenate multiple chains
