@@ -208,7 +208,7 @@ private:
 };
 
 template <typename RecordType = LdltRecords>
-inline std::unique_ptr<McmcSpillover> initialize_spillover(
+inline std::unique_ptr<McmcSpillover> initialize_ctaspillover(
 	int chain_id, int lag, int step, LIST& fit_record, bool sparse, int id = -1,
 	Optional<Eigen::MatrixXd> har_trans = NULLOPT, Optional<int> week = NULLOPT
 ) {
@@ -229,7 +229,7 @@ inline std::unique_ptr<McmcSpillover> initialize_spillover(
 }
 
 template <typename RecordType = LdltRecords>
-inline std::unique_ptr<McmcSpillover> initialize_spillover(
+inline std::unique_ptr<McmcSpillover> initialize_ctaspillover(
 	int lag, int step, RecordType& reg_record, int id = -1,
 	Optional<Eigen::MatrixXd> har_trans = NULLOPT, Optional<int> week = NULLOPT
 ) {
@@ -253,9 +253,9 @@ template <typename RecordType = LdltRecords>
 class McmcSpilloverRun {
 public:
 	McmcSpilloverRun(int lag, int step, LIST& fit_record, bool sparse)
-	: spillover_ptr(initialize_spillover<RecordType>(0, lag, step, fit_record, sparse, -1)) {}
+	: spillover_ptr(initialize_ctaspillover<RecordType>(0, lag, step, fit_record, sparse, -1)) {}
 	McmcSpilloverRun(int week, int month, int step, LIST& fit_record, bool sparse)
-	: spillover_ptr(initialize_spillover<RecordType>(0, month, step, fit_record, sparse, -1, NULLOPT, week)) {}
+	: spillover_ptr(initialize_ctaspillover<RecordType>(0, month, step, fit_record, sparse, -1, NULLOPT, week)) {}
 	virtual ~McmcSpilloverRun() = default;
 	LIST returnSpillover() {
 		return spillover_ptr->returnSpilloverDensity();
@@ -274,6 +274,7 @@ public:
 	DynamicLdltSpillover(
 		const Eigen::MatrixXd& y, int window, int step, int lag, int num_chains, int num_iter, int num_burn, int thin, bool sparse,
 		LIST& param_reg, LIST& param_prior, LIST& param_intercept, LIST_OF_LIST& param_init, int prior_type, bool ggl,
+		LIST& contem_prior, LIST_OF_LIST& contem_init, int contem_prior_type,
 		const Eigen::VectorXi& grp_id, const Eigen::VectorXi& own_id, const Eigen::VectorXi& cross_id, const Eigen::MatrixXi& grp_mat,
 		bool include_mean, const Eigen::MatrixXi& seed_chain, int nthreads
 	)
@@ -288,12 +289,14 @@ public:
 		har_trans(NULLOPT) {
 		initialize(
 			y, param_reg, param_prior, param_intercept, param_init, prior_type, ggl,
+			contem_prior, contem_init, contem_prior_type,
 			grp_id, own_id, cross_id, grp_mat, seed_chain
 		);
 	}
 	DynamicLdltSpillover(
 		const Eigen::MatrixXd& y, int window, int step, int week, int month, int num_chains, int num_iter, int num_burn, int thin, bool sparse,
 		LIST& param_reg, LIST& param_prior, LIST& param_intercept, LIST_OF_LIST& param_init, int prior_type, bool ggl,
+		LIST& contem_prior, LIST_OF_LIST& contem_init, int contem_prior_type,
 		const Eigen::VectorXi& grp_id, const Eigen::VectorXi& own_id, const Eigen::VectorXi& cross_id, const Eigen::MatrixXi& grp_mat,
 		bool include_mean, const Eigen::MatrixXi& seed_chain, int nthreads
 	)
@@ -311,6 +314,7 @@ public:
 		}
 		initialize(
 			y, param_reg, param_prior, param_intercept, param_init, prior_type, ggl,
+			contem_prior, contem_init, contem_prior_type,
 			grp_id, own_id, cross_id, grp_mat,
 			seed_chain
 		);
@@ -356,6 +360,7 @@ protected:
 	 */
 	void initialize(
 		const Eigen::MatrixXd& y, LIST& param_reg, LIST& param_prior, LIST& param_intercept, LIST_OF_LIST& param_init, int prior_type, bool ggl,
+		LIST& contem_prior, LIST_OF_LIST& contem_init, int contem_prior_type,
 		const Eigen::VectorXi& grp_id, const Eigen::VectorXi& own_id, const Eigen::VectorXi& cross_id, const Eigen::MatrixXi& grp_mat,
 		const Eigen::MatrixXi& seed_chain
 	) {
@@ -379,6 +384,7 @@ protected:
 				model[i] = initialize_mcmc<McmcReg, true>(
 					num_chains, num_iter - num_burn, roll_design, roll_y0,
 					param_reg, param_prior, param_intercept, param_init, prior_type,
+					contem_prior, contem_init, contem_prior_type,
 					grp_id, own_id, cross_id, grp_mat,
 					include_mean, seed_chain.row(i)
 				);
@@ -386,6 +392,7 @@ protected:
 				model[i] = initialize_mcmc<McmcReg, false>(
 					num_chains, num_iter - num_burn, roll_design, roll_y0,
 					param_reg, param_prior, param_intercept, param_init, prior_type,
+					contem_prior, contem_init, contem_prior_type,
 					grp_id, own_id, cross_id, grp_mat,
 					include_mean, seed_chain.row(i)
 				);
@@ -406,7 +413,7 @@ protected:
 			model[window][chain]->doPosteriorDraws();
 		}
 		LdltRecords reg_record = model[window][chain]->returnLdltRecords(0, thin, sparse);
-		spillover[window][chain] = initialize_spillover<LdltRecords>(lag, step, reg_record, -1, har_trans);
+		spillover[window][chain] = initialize_ctaspillover<LdltRecords>(lag, step, reg_record, -1, har_trans);
 		model[window][chain].reset();
 	}
 	void getSpillover(int window, int chain) {
@@ -492,7 +499,7 @@ protected:
 		#pragma omp parallel for num_threads(nthreads)
 	#endif
 		for (int window = 0; window < num_horizon; ++window) {
-			spillover[window] = initialize_spillover<SvRecords>(lag, step, *reg_record, window, har_trans);
+			spillover[window] = initialize_ctaspillover<SvRecords>(lag, step, *reg_record, window, har_trans);
 			spillover[window]->computeSpillover();
 			to_sp[window] = spillover[window]->returnTo();
 			from_sp[window] = spillover[window]->returnFrom();
