@@ -8,7 +8,10 @@ test_that("Test for VAR-type minnesota", {
     lambda = .2,
     delta = rep(.1, etf_ncol)
   )
-  fit_test_bvhar_var <- bvhar_minnesota(etf_vix, bayes_spec = var_spec)
+  fit_test_bvhar_var <- bvhar_minnesota(
+    y = etf_vix,
+    bayes_spec = var_spec
+  )
   
   expect_s3_class(var_spec, "bvharspec")
   expect_s3_class(fit_test_bvhar_var, "bvharmn")
@@ -35,7 +38,12 @@ test_that("Test for VHAR-type minnesota", {
     weekly = rep(.2, etf_ncol),
     monthly = rep(.1, etf_ncol)
   )
-  fit_test_bvhar_vhar <- bvhar_minnesota(etf_vix, bayes_spec = vhar_spec)
+  fit_test_bvhar_vhar <- bvhar_minnesota(
+    y = etf_vix,
+    num_iter = 10,
+    num_burn = 0,
+    bayes_spec = vhar_spec
+  )
   
   expect_s3_class(vhar_spec, "bvharspec")
   expect_s3_class(fit_test_bvhar_vhar, "bvharmn")
