@@ -33,86 +33,143 @@ test_that("Rolling windows - OLS", {
 
 test_that("Expanding windows - OLS", {
   skip_on_cran()
-  
+
   etf_split <- divide_ts(etf_vix[1:100, 1:2], 10)
   etf_train <- etf_split$train
   etf_test <- etf_split$test
-  
+
   fit_var <- var_lm(etf_train, 2)
   var_expand <- forecast_expand(fit_var, 10, etf_test)
   expect_s3_class(var_expand, "predbvhar_expand")
   expect_s3_class(var_expand, "bvharcv")
-  
 })
 
-help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse) {
+test_that("Rolling windows - OLS VHAR", {
+  skip_on_cran()
+
+  etf_split <- divide_ts(etf_vix[1:100, 1:2], 10)
+  etf_train <- etf_split$train
+  etf_test <- etf_split$test
+
+  fit_test <- vhar_lm(etf_train)
+  var_roll <- forecast_roll(fit_test, 10, etf_test)
+  expect_s3_class(var_roll, "predbvhar_roll")
+  expect_s3_class(var_roll, "bvharcv")
+})
+
+test_that("Expanding windows - OLS VHAR", {
+  skip_on_cran()
+
+  etf_split <- divide_ts(etf_vix[1:100, 1:2], 10)
+  etf_train <- etf_split$train
+  etf_test <- etf_split$test
+
+  fit_test <- vhar_lm(etf_train)
+  var_expand <- forecast_expand(fit_test, 10, etf_test)
+  expect_s3_class(var_expand, "predbvhar_expand")
+  expect_s3_class(var_expand, "bvharcv")
+})
+
+help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
+  etf_exogen_train <- NULL
+  etf_exogen_test <- NULL
+  if (is_exogen) {
+    etf_exogen_train <- etf_vix[1:50, 3:4]
+    etf_exogen_test <- etf_vix[51:53, 3:4]
+  }
 
   set.seed(1)
   fit_test <- var_bayes(
     etf_train,
     p = 1,
+    exogen = etf_exogen_train,
     num_iter = 3,
     num_burn = 0,
-    bayes_spec = bayes_spec,
+    coef_spec = bayes_spec,
+    contem_spec = bayes_spec,
     cov_spec = cov_spec,
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_roll(fit_test, 1, etf_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
 }
 
-help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse) {
+help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
+  etf_exogen_train <- NULL
+  etf_exogen_test <- NULL
+  if (is_exogen) {
+    etf_exogen_train <- etf_vix[1:50, 3:4]
+    etf_exogen_test <- etf_vix[51:53, 3:4]
+  }
 
   set.seed(1)
   fit_test <- vhar_bayes(
     etf_train,
+    exogen = etf_exogen_train,
     num_iter = 3,
     num_burn = 0,
-    bayes_spec = bayes_spec,
+    coef_spec = bayes_spec,
+    contem_spec = bayes_spec,
     cov_spec = cov_spec,
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_roll(fit_test, 1, etf_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
 }
 
-help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse) {
+help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
+  etf_exogen_train <- NULL
+  etf_exogen_test <- NULL
+  if (is_exogen) {
+    etf_exogen_train <- etf_vix[1:50, 3:4]
+    etf_exogen_test <- etf_vix[51:53, 3:4]
+  }
 
   set.seed(1)
   fit_test <- var_bayes(
     etf_train,
     p = 1,
+    exogen = etf_exogen_train,
     num_iter = 3,
     num_burn = 0,
-    bayes_spec = bayes_spec,
+    coef_spec = bayes_spec,
+    contem_spec = bayes_spec,
     cov_spec = cov_spec,
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_expand(fit_test, 1, etf_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
 }
 
-help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse) {
+help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
+  etf_exogen_train <- NULL
+  etf_exogen_test <- NULL
+  if (is_exogen) {
+    etf_exogen_train <- etf_vix[1:50, 3:4]
+    etf_exogen_test <- etf_vix[51:53, 3:4]
+  }
 
   set.seed(1)
   fit_test <- vhar_bayes(
     etf_train,
+    exogen = etf_exogen_train,
     num_iter = 3,
     num_burn = 0,
-    bayes_spec = bayes_spec,
+    coef_spec = bayes_spec,
+    contem_spec = bayes_spec,
     cov_spec = cov_spec,
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_expand(fit_test, 1, etf_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
 }
 
 test_that("Rolling windows - VAR-Minn-LDLT", {
@@ -120,6 +177,8 @@ test_that("Rolling windows - VAR-Minn-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_bvar(), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_bvar(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_bvar(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_bvar(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -133,6 +192,8 @@ test_that("Rolling windows - VAR-HS-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -146,6 +207,8 @@ test_that("Rolling windows - VAR-SSVS-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_ssvs(), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_ssvs(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_ssvs(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_ssvs(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -159,6 +222,8 @@ test_that("Rolling windows - VAR-Hierminn-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -172,6 +237,8 @@ test_that("Rolling windows - VAR-NG-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_ng(), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_ng(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_ng(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_ng(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -185,6 +252,8 @@ test_that("Rolling windows - VAR-DL-LDLT", {
 
   test_roll_dense <- help_var_bayes_roll(set_dl(), set_ldlt(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_dl(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_dl(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_dl(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -198,6 +267,8 @@ test_that("Rolling windows - VHAR-Minn-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -211,6 +282,8 @@ test_that("Rolling windows - VHAR-HS-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_horseshoe(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_horseshoe(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_horseshoe(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_horseshoe(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -224,6 +297,8 @@ test_that("Rolling windows - VHAR-SSVS-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_ssvs(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_ssvs(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_ssvs(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_ssvs(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -237,6 +312,8 @@ test_that("Rolling windows - VHAR-Hierminn-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -250,6 +327,8 @@ test_that("Rolling windows - VHAR-NG-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_ng(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_ng(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_ng(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_ng(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -263,6 +342,8 @@ test_that("Rolling windows - VHAR-DL-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_dl(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_dl(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_dl(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_dl(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -276,6 +357,8 @@ test_that("Rolling windows - VHAR-GDP-LDLT", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_gdp(), set_ldlt(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_gdp(), set_ldlt(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_gdp(), set_ldlt(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_gdp(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -289,6 +372,8 @@ test_that("Rolling windows - VAR-Minn-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_bvar(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_bvar(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_bvar(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_bvar(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -302,6 +387,8 @@ test_that("Rolling windows - VAR-HS-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_horseshoe(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_horseshoe(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_horseshoe(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_horseshoe(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -315,6 +402,8 @@ test_that("Rolling windows - VAR-SSVS-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_ssvs(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_ssvs(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_ssvs(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_ssvs(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -328,6 +417,8 @@ test_that("Rolling windows - VAR-Hierminn-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_bvar(lambda = set_lambda()), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -341,6 +432,8 @@ test_that("Rolling windows - VAR-NG-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_ng(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_ng(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_ng(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_ng(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -354,6 +447,8 @@ test_that("Rolling windows - VAR-DL-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_dl(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_dl(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_dl(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_dl(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -367,6 +462,8 @@ test_that("Rolling windows - VAR-GDP-SV", {
 
   test_roll_dense <- help_var_bayes_roll(set_gdp(), set_sv(), FALSE)
   test_roll_sparse <- help_var_bayes_roll(set_gdp(), set_sv(), TRUE)
+  test_roll_dense_x <- help_var_bayes_roll(set_gdp(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_var_bayes_roll(set_gdp(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -380,6 +477,8 @@ test_that("Rolling windows - VHAR-Minn-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_bvhar(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_bvhar(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_bvhar(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_bvhar(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -393,6 +492,8 @@ test_that("Rolling windows - VHAR-HS-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_horseshoe(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_horseshoe(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_horseshoe(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_horseshoe(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -406,6 +507,8 @@ test_that("Rolling windows - VHAR-SSVS-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_ssvs(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_ssvs(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_ssvs(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_ssvs(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -419,6 +522,8 @@ test_that("Rolling windows - VHAR-Hierminn-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_bvhar(lambda = set_lambda()), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -432,6 +537,8 @@ test_that("Rolling windows - VHAR-NG-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_ng(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_ng(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_ng(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_ng(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -445,6 +552,8 @@ test_that("Rolling windows - VHAR-DL-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_dl(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_dl(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_dl(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_dl(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -458,6 +567,8 @@ test_that("Rolling windows - VHAR-GDP-SV", {
 
   test_roll_dense <- help_vhar_bayes_roll(set_gdp(), set_sv(), FALSE)
   test_roll_sparse <- help_vhar_bayes_roll(set_gdp(), set_sv(), TRUE)
+  test_roll_dense_x <- help_vhar_bayes_roll(set_gdp(), set_sv(), FALSE, TRUE)
+  test_roll_sparse_x <- help_vhar_bayes_roll(set_gdp(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -471,6 +582,8 @@ test_that("Expanding windows - VAR-Minn-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_bvar(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_bvar(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_bvar(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_bvar(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -484,6 +597,8 @@ test_that("Expanding windows - VAR-HS-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_horseshoe(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_horseshoe(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_horseshoe(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_horseshoe(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -497,6 +612,8 @@ test_that("Expanding windows - VAR-SSVS-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_ssvs(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_ssvs(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_ssvs(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_ssvs(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -510,6 +627,8 @@ test_that("Expanding windows - VAR-Hierminn-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -523,6 +642,8 @@ test_that("Expanding windows - VAR-NG-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_ng(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_ng(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_ng(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_ng(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -536,6 +657,8 @@ test_that("Expanding windows - VAR-DL-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_dl(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_dl(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_dl(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_dl(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -549,6 +672,8 @@ test_that("Expanding windows - VAR-GDP-LDLT", {
 
   test_expand_dense <- help_var_bayes_expand(set_gdp(), set_ldlt(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_gdp(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_gdp(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_gdp(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -562,6 +687,8 @@ test_that("Expanding windows - VHAR-Minn-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -575,6 +702,8 @@ test_that("Expanding windows - VHAR-HS-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_horseshoe(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_horseshoe(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_horseshoe(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_horseshoe(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -588,6 +717,8 @@ test_that("Expanding windows - VHAR-SSVS-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_ssvs(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_ssvs(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_ssvs(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_ssvs(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -601,6 +732,8 @@ test_that("Expanding windows - VHAR-Hierminn-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -614,6 +747,8 @@ test_that("Expanding windows - VHAR-NG-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_ng(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_ng(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_ng(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_ng(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -627,6 +762,8 @@ test_that("Expanding windows - VHAR-DL-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_dl(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_dl(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_dl(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_dl(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -640,6 +777,8 @@ test_that("Expanding windows - VHAR-GDP-LDLT", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_gdp(), set_ldlt(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_gdp(), set_ldlt(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_gdp(), set_ldlt(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_gdp(), set_ldlt(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -653,6 +792,8 @@ test_that("Expanding windows - VAR-Minn-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_bvar(), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_bvar(), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_bvar(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_bvar(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -666,6 +807,8 @@ test_that("Expanding windows - VAR-HS-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_horseshoe(), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_horseshoe(), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_horseshoe(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_horseshoe(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -679,6 +822,8 @@ test_that("Expanding windows - VAR-SSVS-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_ssvs(), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_ssvs(), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_ssvs(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_ssvs(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -692,6 +837,8 @@ test_that("Expanding windows - VAR-Hierminn-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_bvar(lambda = set_lambda()), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -705,6 +852,8 @@ test_that("Expanding windows - VAR-NG-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_ng(), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_ng(), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_ng(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_ng(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -718,6 +867,8 @@ test_that("Expanding windows - VAR-DL-SV", {
 
   test_expand_dense <- help_var_bayes_expand(set_dl(), set_sv(), FALSE)
   test_expand_sparse <- help_var_bayes_expand(set_dl(), set_sv(), TRUE)
+  test_expand_dense_x <- help_var_bayes_expand(set_dl(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_var_bayes_expand(set_dl(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -731,6 +882,8 @@ test_that("Expanding windows - VHAR-Minn-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_bvhar(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_bvhar(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_bvhar(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_bvhar(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -744,6 +897,8 @@ test_that("Expanding windows - VHAR-HS-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_horseshoe(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_horseshoe(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_horseshoe(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_horseshoe(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -757,6 +912,8 @@ test_that("Expanding windows - VHAR-SSVS-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_ssvs(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_ssvs(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_ssvs(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_ssvs(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -770,6 +927,8 @@ test_that("Expanding windows - VHAR-Hierminn-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_bvhar(lambda = set_lambda()), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -783,6 +942,8 @@ test_that("Expanding windows - VHAR-NG-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_ng(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_ng(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_ng(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_ng(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -796,6 +957,8 @@ test_that("Expanding windows - VHAR-DL-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_dl(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_dl(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_dl(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_dl(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -809,6 +972,8 @@ test_that("Expanding windows - VHAR-GDP-SV", {
 
   test_expand_dense <- help_vhar_bayes_expand(set_gdp(), set_sv(), FALSE)
   test_expand_sparse <- help_vhar_bayes_expand(set_gdp(), set_sv(), TRUE)
+  test_expand_dense_x <- help_vhar_bayes_expand(set_gdp(), set_sv(), FALSE, TRUE)
+  test_expand_sparse_x <- help_vhar_bayes_expand(set_gdp(), set_sv(), TRUE, TRUE)
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
