@@ -3,7 +3,7 @@ test_that("Test for varlse class", {
   skip_on_cran()
   
   test_lag <- 3
-  fit_test_var <- var_lm(etf_vix, test_lag)
+  fit_test_var <- var_lm(y = etf_vix, p = test_lag)
   num_col <- ncol(etf_vix)
   num_row <- nrow(etf_vix)
 
@@ -39,9 +39,33 @@ test_that("Computation Methods", {
   skip_on_cran()
   
   test_lag <- 3
-  fit_test_nor <- var_lm(etf_vix[, 1:3], test_lag)
-  fit_test_llt <- var_lm(etf_vix[, 1:3], test_lag, method = "chol")
-  fit_test_qr <- var_lm(etf_vix[, 1:3], test_lag, method = "qr")
+  fit_test_nor <- var_lm(y = etf_vix[, 1:3], p = test_lag)
+  fit_test_llt <- var_lm(y = etf_vix[, 1:3], p = test_lag, method = "chol")
+  fit_test_qr <- var_lm(y = etf_vix[, 1:3], p = test_lag, method = "qr")
+
+  expect_equal(
+    fit_test_nor$coefficients,
+    fit_test_llt$coefficients
+  )
+
+  expect_equal(
+    fit_test_nor$coefficients,
+    fit_test_qr$coefficients
+  )
+})
+
+test_that("VARX", {
+  skip_on_cran()
+
+  test_lag <- 3
+  test_lag_exogen <- 2
+  
+  endog_y <- etf_vix[, 1:3]
+  exogen <- etf_vix[,4:5]
+  
+  fit_test_nor <- var_lm(y = endog_y, p = test_lag, exogen = exogen, s = test_lag_exogen)
+  fit_test_llt <- var_lm(y = endog_y, p = test_lag, exogen = exogen, s = test_lag_exogen, method = "chol")
+  fit_test_qr <- var_lm(y = endog_y, p = test_lag, exogen = exogen, s = test_lag_exogen, method = "qr")
 
   expect_equal(
     fit_test_nor$coefficients,
